import {Component, createElement, PropTypes} from 'react';
import hoistStatics from 'hoist-non-react-statics';
import {connect} from 'react-redux';

import {getList} from './selector';
import {hide} from './action';
import {contextShape, portalItemShape, getDisplayName} from './util';

export default (WrappedComponent) => {
  class Connect extends Component {
    static propTypes = {
      list: PropTypes.arrayOf(portalItemShape),
    };

    static contextTypes = {
      portal: contextShape,
    };

    static defaultProps = {
      list: [],
    };

    render() {
      const mergedProps = {
        ...this.props,
        list: [
          // Prepend items collected from the `<PortalProvider>` to the list
          // of items collected from the redux store.
          ...this.context.portal.list || [],

          // Assign default hide functions to portals from the redux store.
          ...this.props.list
            .map((item) => ({
              hide: () => this.props.hideItem(item.id),
              ...item,
            })),
        ],
      };

      return createElement(WrappedComponent, mergedProps);
    }
  }

  Connect.displayName = `ReduxPortal(${getDisplayName(WrappedComponent)})`;

  const mapState = (state, props) => ({portals: getList(state, props)});
  const mapDispatch = (dispatch) => ({hideItem: (id) => dispatch(hide(id))});

  return connect(
    mapState,
    mapDispatch
  )(hoistStatics(Connect, WrappedComponent));
};
