import {Component, createElement, PropTypes} from 'react';
import {formatPattern, routerShape} from 'react-router';
import hoistStatics from 'hoist-non-react-statics';

import PortalProvider from './provider';

const getDisplayName = (C) => C.displayName || C.name || 'Component';

export const createRouterComponent = (WrappedComponent) => {
  class Connect extends Component {
    static propTypes = {
      routes: PropTypes.array.isRequired,
      params: PropTypes.object.isRequired,
    };

    static contextTypes = {
      router: routerShape,
    };

    constructor(state, props) {
      super(state, props);
      this.state = {previousLocation: {}};
    }

    componentWillReceiveProps() {
      this.setState({previousLocation: this.props.location});
    }

    render() {
      const {routes, params} = this.props;
      const {router} = this.context;

      // Create a function that will hide the item by navigating to a different
      // path. Call `router.replace` with `item.hide` if it is defined,
      // otherwise call router.replace with the formatted path of the parent
      // route.
      const createHide = (i) => {
        const item = routes[i].portal;

        let path = item.hide;

        if (!path) {
          let parentPath;

          for (let current = i - 1; !parentPath && i >= 0; current--) {
            const parent = routes[current];
            parentPath = parent && parent.path;
          }

          if (parentPath) {
            path = formatPattern(parentPath, params);
          }
        }

        if (!path) {
          return null;
        }

        return () => {
          if (path === this.state.previousLocation.pathname) {
            router.goBack();
          } else {
            router.push(path);
          }
        };
      };

      const list = routes.reduce((list, route, i) => {
        const {portal, path} = route;

        if (portal) {
          list.push({
            // Use the formatted path that matched the portal as the default id.
            id: formatPattern(path, params),

            // Merge portal item props with route component props.
            props: {...this.props, ...portal.props},

            // If `portal` is a string, assign it the type. Otherwise merge it.
            ...typeof portal === 'string' ? {type: portal} : portal,

            // Assign a hide function. Use `portal.hide` if it's a function.
            hide: typeof portal.hide === 'function' ?
              portal.hide :
              createHide(i),
          });
        }

        return list;
      }, []);

      return createElement(WrappedComponent, {list, ...this.props});
    }
  }

  Connect.displayName = `PortalRouter(${getDisplayName(WrappedComponent)})`;

  return hoistStatics(Connect, WrappedComponent);
};

export default createRouterComponent(PortalProvider);
